﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Management;
using System.Text.RegularExpressions;

using Microsoft.VisualBasic;
using System.Diagnostics; 

namespace DCS_9700_Wave
{
    public partial class Form_Wave : Form
    {
        public Form_Wave()
        {
            InitializeComponent();

            //Button clear
            cmbCOMPorts.Enabled = true;
            btnCloseDev.Enabled = false;
            btnRead.Enabled = false; 
            btnSave.Enabled = false;
            btnFFF.Enabled = false; 
            BtnOpenDev.Enabled = true;

            // COM port Listing
            cmbCOMPorts.Items.Clear();

            //USB変換が表示されないので修正
/*            ManagementClass mcW32SerPort = new ManagementClass("Win32_SerialPort");
            foreach (ManagementObject aSerialPort in mcW32SerPort.GetInstances())
            {
                cmbCOMPorts.Items.Add((string)aSerialPort.GetPropertyValue("DeviceID") + ": " + (string)aSerialPort.GetPropertyValue("Caption"));
            }
*/

            List<string> portlist = new List<string>(System.IO.Ports.SerialPort.GetPortNames());
            //ManagementObjectCollection moc = new ManagementClass("Win32_PnPEntity").GetInstances(); Win10ではNG
            ManagementObjectCollection moc = new ManagementClass("Win32_SerialPort").GetInstances();

            var b =
                from string str in portlist
                from ManagementObject mo in moc
                let devName = mo["Name"].ToString()
                let devID = mo["DeviceID"].ToString()
                where Regex.IsMatch(devName, str + "[^0-9]")
                select new { str, devName};

            foreach (var pn in b)
            {
                cmbCOMPorts.Items.Add(pn.str + ": " + pn.devName);
            }

            cmbCOMPorts.SelectedIndex = 0;
        }

        /// <summary>
        /// Open Device port
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnOpenDev_Click(object sender, EventArgs e)
        {
            //msg clear
            txtReadBuf.Text = "";
            txtReadWave.Text = "";

            // Port Setting 
            string[] strPortDesc = cmbCOMPorts.Text.Split(':');
            serialPort1.PortName = strPortDesc[0] ;
            if (chkCDCClass.Checked == false)
            {
                serialPort1.BaudRate = 115200;      //Serial Port Please Set DCS-9700 to 115200bps
            }
            else
            {
                serialPort1.BaudRate = 9600;        // USB CDC Class
            }
            serialPort1.DataBits = 8;
            serialPort1.Parity = System.IO.Ports.Parity.None;
            serialPort1.StopBits = System.IO.Ports.StopBits.One;
            serialPort1.Handshake = System.IO.Ports.Handshake.None ;
            serialPort1.ReadTimeout = 3000;
            serialPort1.WriteTimeout = 500;
            serialPort1.ReadBufferSize = 10600;
            serialPort1.NewLine = "\n";
            serialPort1.Open();
            serialPort1.DiscardInBuffer();
            cmbCOMPorts.Enabled = false;

            // IDN query
            serialPort1.WriteLine("*idn?");
            System.Threading.Thread.Sleep(50);
            try
            {
                txtReadBuf.Text = serialPort1.ReadLine();

                //btn setting
                BtnOpenDev.Enabled = false; 
                btnCloseDev.Enabled = true;
                btnRead.Enabled = true;
                btnSave.Enabled = true;
                btnFFF.Enabled = true;
            }
            catch (System.Exception caught)
            {
                MessageBox.Show(caught.Message, "Open Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                serialPort1.Close();
                cmbCOMPorts.Enabled = true;
                btnCloseDev.Enabled = false;
                btnRead.Enabled = false;
                btnSave.Enabled = false; 
                btnFFF.Enabled = false;
                BtnOpenDev.Enabled = true;
            }

        }

        /// <summary>
        /// Close Device Port
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCloseDev_Click(object sender, EventArgs e)
        {
            serialPort1.Close();
            cmbCOMPorts.Enabled = true;
            btnCloseDev.Enabled = false;
            btnRead.Enabled = false;
            btnSave.Enabled = false;
            btnFFF.Enabled = false;
            BtnOpenDev.Enabled = true;
        }

        
        /// <summary>
        /// Read Wave Data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRead_Click(object sender, EventArgs e)
        {
            string tmpstr = "";
            int iTemp;
            byte[] buf = new byte[10001];
            int[] sbuf = new int[5000];
            serialPort1.WriteLine(":acq1:mem?");
            System.Threading.Thread.Sleep(50);

            // Wave Header read
            try
            {
                tmpstr = serialPort1.ReadLine();
            }
            catch (System.Exception caught)
            {
                MessageBox.Show(caught.Message, "Open Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                tmpstr = "";
            }

            txtReadBuf.Text = tmpstr;
            txtReadWave.Text  = "" ;
            System.Threading.Thread.Sleep(300);
            if (chkCDCClass.Checked == false)
            {
                System.Threading.Thread.Sleep(1000);
            }
            // Wave data Read
            try
            {
                serialPort1.Read(buf, 0, 7);        // Binary Header
                serialPort1.Read(buf, 0, 10000);    // Main data
                serialPort1.DiscardInBuffer();
            }
            catch (System.Exception caught)
            {
                MessageBox.Show(caught.Message, "Open Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            tmpstr = "";
            // Data Convert  Horizontal 5000point :10div ,Vertical 250point :10div
            for (int index = 0; index < 5000; index++)
            {
                // Endian convert (byteswap)
                iTemp = buf[index * 2];
                buf[index * 2] = buf[index * 2 + 1];
                buf[index * 2 + 1] = (byte)iTemp;
                sbuf[index] = BitConverter.ToInt16(buf,index * 2);
                tmpstr = tmpstr + "\r\n" + sbuf[index];
            }
            txtReadWave.Text = tmpstr + "\r\n";

            //Create object BMP,Graphics 
             Bitmap canvas = new Bitmap(pictureBox1.Width, pictureBox1.Height);
             Graphics g = Graphics.FromImage(canvas);

            //Plot to image
            for (int index = 0; index < 4999; index++)
                g.DrawLine(Pens.OrangeRed , index/20, (130-sbuf[index])/2, index/20, (130-sbuf[index+1])/2);

            //Display image
            g.Dispose();
            pictureBox1.Image = canvas;
        }

        /// <summary>
        /// Save wave data to file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Title = "Save Wave data to File";
            saveFileDialog1.Filter = "TEXT File|*.txt|All File|*.*";
            saveFileDialog1.OverwritePrompt = true;
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                System.IO.StreamWriter writer = new System.IO.StreamWriter(saveFileDialog1.FileName, false, Encoding.GetEncoding("Shift_JIS"));
                writer.Write(txtReadBuf.Text);
                writer.Write(txtReadWave.Text);
                writer.Close();
            }
            saveFileDialog1.Dispose();
        }

        private void btnFFF_Click(object sender, EventArgs e)
        {

            string tmpstr = "";
            int iTemp;
            txtReadBuf.Text = "";
            btnFFF.Enabled = false;

            for (iTemp = 0; iTemp < 5; iTemp++)
            {
                try
                {
                    serialPort1.WriteLine(":meas:fffd?");
                    System.Threading.Thread.Sleep(50);
                    txtReadBuf.Text =  String.Format("{0, -4}  ", iTemp)+ serialPort1.ReadLine() + "\r\n" + txtReadBuf.Text ;
                }
                catch (System.Exception caught)
                {
                    MessageBox.Show(caught.Message, "Open Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                Application.DoEvents();
                System.Threading.Thread.Sleep(100);
            }
            btnFFF.Enabled = true ;
        }
    }
}
